import numpy as np
import Nio as nio
import Ngl as ngl

#----------------------------------------------------------------------
# Function to plot WRF variable
#----------------------------------------------------------------------
def wrf_plot_var(var,lat=None,lon=None,output_type="png",
                 output_name="wrfplot",res=None,
                 contour_type="default",title=None,
                 color_map="ncl_default"):
  """
Creates a color contour plot of the given variable. If lat/lon are
provided, or any "mp" resources are set, then the contours will be
drawn over a map.

If variable is more than two dimensions, then only the first element
of the leftmost dimensions are plotted.

wrf_plot_from_var(x)

var          - The variable to plot
filename     - The name of the WRF output file
output_type  - The type of output file or window to send the graphics to
                   ("png","x11","ps","pdf","svg")
output_name  - The prefix name to give the output ("wrfplot")
res          - Optional resources to set
title        - Optional title to include on plot
contour_type - "default" (smoothed file), "raster", "cell"
color_map    - The color map to use 
  """

  wks = _get_output(output_type,output_name)
  r   = _create_res_object(res)
  _add_latlon_and_zoom_res(r,lat,lon)
  _add_contour_res(r,color_map,contour_type,title)
  _draw_plot(wks,var,r)

#----------------------------------------------------------------------
# Function to plot WRF variable from WRF file.
#----------------------------------------------------------------------
def wrf_plot_from_file(filename,vname,output_type="png",output_name="wrfplot",
                       res=None,contour_type="default",color_map="ncl_default"):
  """
Creates a color contour plot over a map of the given variable on the given 
WRF output file, using the native projection defined on the WRF file.

If variable is more than two dimensions, then only the first element
of the leftmost dimensions are plotted.

wrf_plot_from_file("wrfout_d03_2012-04-22_23_00_00.nc","T")

filename     - The name of the WRF output file
vname        - The variable name to read off file and plot
output_type  - The type of output file or window to send the graphics to
                  ("png","x11","ps","pdf","svg")
output_name  - The prefix name to give the output ("wrfplot")
res          - Optional resources to set
contour_type - "default" (smoothed file), "raster", "cell"
  """

# Open the file
  if filename.endswith(".nc"):
    a = nio.open_file(filename)
  else:
    a = nio.open_file(filename + ".nc")

# Check for variable name on file
  vnames = a.variables.keys()
  if(not vname in vnames):
    print("wrf_plot_from_file: error, %s not found in %s" % (vname,filename))
    return

# Construct a plot title
  if hasattr(a.variables[vname],"description") and hasattr(a.variables[vname],"units"):
    title = "%s (%s)" % (a.variables[vname].description,a.variables[vname].units)
  else:
    title = "%s" % vname

  var = _read_var(a,vname)
  r   = _create_res_object(res)
  ngl.wrf_map_resources(filename,r)   # Add WRF native projection map options
  r.mpGeophysicalLineThicknessF = 2.0
  r.mpGeophysicalLineColor      = "black"
  r.mpNationalLineThicknessF    = 2.0
  r.mpNationalLineColor         = "black"
  r.mpUSStateLineColor          = "black"
  r.mpUSStateLineThicknessF     = 2.0

  wrf_plot_var(var,output_name=output_name,output_type=output_type,res=r,
               contour_type=contour_type,color_map=color_map,title=title)

#----------------------------------------------------------------------
# Function to read 2D variable from Nio file object.
#----------------------------------------------------------------------
def _read_var(a,vname):
  rank = len(a.variables[vname].shape)
  if(rank == 2):
    var = a.variables[vname][:]
  elif(rank == 3):
    var = a.variables[vname][0,:,:]
  elif(rank == 4):
    var = a.variables[vname][0,0,:,:]
  elif(rank == 5):
    var = a.variables[vname][0,0,0,:,:]
  return var

#----------------------------------------------------------------------
# Function to return object for graphical output
#----------------------------------------------------------------------
def _get_output(output_type,output_name=None):
  otype = output_type
  oname = output_name

  if(otype not in ["x11","png","pdf","ps","svg"]):
    print("wrf_plot_from_var: error, don't recognize output type of %s" % otype)
    print("          Defaulting to png.")
    otype = "png"

  if oname is None:
    oname = "wrfplot"

  if otype is "x11":
    print "Output will go to an X11 window"
  else:
    print "Output will go to a file called %s.%s" % (oname,otype)

  if otype is "png":
    wres          = ngl.Resources()
    wres.wkWidth  = 500                # default is larger
    wres.wkHeight = 500
    wks = ngl.open_wks(otype,oname,wres)
  else:
    wks = ngl.open_wks(otype,oname)

  return(wks)

#----------------------------------------------------------------------
# Function to draw contours or contours over a map
#----------------------------------------------------------------------
def _draw_plot(wks,var,res):
  for att in dir(res):
    if "mp" in att:
      ngl.contour_map(wks,var,res)
      return
  ngl.contour(wks,var,res)
  del wks  # This closes the file

  return

#----------------------------------------------------------------------
# Procedure to check for lat/lon and add resources for zooming
# in on map if available.
#----------------------------------------------------------------------
def _add_latlon_and_zoom_res(res,lat,lon):
  if not lat is None and not lon is None:
    res.sfYArray    = lat
    res.sfXArray    = lon
    res.mpLimitMode = getattr(res,"mpLimitMode","LatLon")
    res.mpMinLatF   = getattr(res,"mpMinLatF",np.min(lat))
    res.mpMaxLatF   = getattr(res,"mpMaxLatF",np.max(lat))
    res.mpMinLonF   = getattr(res,"mpMinLonF",np.min(lon))
    res.mpMaxLonF   = getattr(res,"mpMaxLonF",np.max(lon))

#----------------------------------------------------------------------
# Procedure to add some contour plotting options
#----------------------------------------------------------------------
def _add_contour_res(res,color_map,contour_type,title):
  res.cnFillOn       = getattr(res,"cnFillOn",True)           # turn on contour fill
  res.cnLinesOn      = getattr(res,"cnLinesOn",False)         # turn off contour lines
  res.cnLineLabelsOn = getattr(res,"cnLineLabelsOn",False)    # turn off line labels
  res.cnFillPalette  = getattr(res,"cnFillPalette",color_map) # change color map

# Raster plotting can sometimes be faster.
  if(contour_type == "raster"):
    res.cnFillMode = getattr(res,"cnFillMode", "RasterFill")  
    res.trGridType = getattr(res,"trGridType", "TriangularMesh") 
  elif(contour_type == "cell"):
    res.cnFillMode = getattr(res,"cnFillMode", "CellFill")

  res.lbLabelFontHeightF = getattr(res,"lbLabelFontHeightF", 0.015)  # default is large
  if not title is None:
    res.tiMainString = getattr(res,"tiMainString", title)

  res.nglPointTickmarksOutward = True

#----------------------------------------------------------------------
# Function to create a resource object.
#----------------------------------------------------------------------
def _create_res_object(res):
# Set some plotting options
  if(res is None):
    return ngl.Resources()
  else:
    return res

